@@ -1,8 +1,8 @@
 /* alpha.c - core analysis suite
  *
  * Copyright (C) 1999, 2000, 2001, 2002 Mission Critical Linux, Inc.
- * Copyright (C) 2002, 2003, 2004 David Anderson
- * Copyright (C) 2002, 2003, 2004 Red Hat, Inc. All rights reserved.
+ * Copyright (C) 2002, 2003, 2004, 2005 David Anderson
+ * Copyright (C) 2002, 2003, 2004, 2005 Red Hat, Inc. All rights reserved.
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -14,15 +14,6 @@
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  * GNU General Public License for more details.
  *
- * 02/29/00, 2.2    Bug fixes, new commands, options
- * 04/11/00, 2.3    Bug fixes, new command, options, initial PowerPC framework
- * 04/12/00  ---    Transition to BitKeeper version control
- * 
- * BitKeeper ID: @(#)alpha.c 1.21
- *
- * 09/28/00  ---    Transition to CVS version control
- *
- * CVS: $Revision: 1.30 $ $Date: 2004/11/03 16:35:31 $
  */ 
 #ifdef ALPHA
 #include "defs.h"
@@ -2602,6 +2593,8 @@ alpha_verify_symbol(const char *name, ulong value, char type)
 int
 alpha_get_smp_cpus(void)
 {
+	int cpus;
+
         if ((cpus = get_cpus_online()))
                 return cpus;
         else
