@@ -3,8 +3,8 @@
  * Copyright (C) 1999, 2000, 2001, 2002 Mission Critical Linux, Inc.
  * Copyright (C) 2002 Silicon Graphics, Inc. 
  * Copyright (C) 2002 Free Software Foundation, Inc.
- * Copyright (C) 2002, 2003, 2004 David Anderson
- * Copyright (C) 2002, 2003, 2004 Red Hat, Inc. All rights reserved.
+ * Copyright (C) 2002, 2003, 2004, 2005 David Anderson
+ * Copyright (C) 2002, 2003, 2004, 2005 Red Hat, Inc. All rights reserved.
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -15,20 +15,6 @@
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  * GNU General Public License for more details.
- *
- * 12/10/99, 1.1    Fixes, new commands, support for v1 SGI dumps
- * 01/18/00, 2.0    Initial gdb merger, support for Alpha
- * 02/01/00, 2.1    Bug fixes, new commands, options, support for v2 SGI dumps
- * 02/29/00, 2.2    Bug fixes, new commands, options
- * 04/11/00, 2.3    Bug fixes, new command, options, initial PowerPC framework
- * 04/12/00  ---    Transition to BitKeeper version control
- * 04/17/02  ---    Integration to LKCD and non-monotonically increasing dumps
- * 
- * BitKeeper ID: @(#)sgi_lkcd.c 1.3
- *
- * 09/28/00  ---    Transition to CVS version control
- *
- * CVS: $Revision: 1.17 $ $Date: 2004/10/15 19:59:23 $
  */
 
 /*
@@ -1085,7 +1071,8 @@ cache_page(void)
 		case LKCD_DUMP_COMPRESS_GZIP:
 			if (!lkcd_uncompress_gzip((unsigned char *)
 			    lkcd->page_cache_hdr[i].pg_bufptr, lkcd->page_size,
-			    lkcd->compressed_page, lkcd->get_dp_size())) {
+			    (unsigned char *)lkcd->compressed_page, 
+			    lkcd->get_dp_size())) {
                                 lkcd_print("uncompress of page ");
                                 lkcd_print(BITS32() ? 
                                         "%llx failed!\n" : "%lx failed!\n",
